package ecp.vcs.com.ecpsyncplugin.addons.addons_plugin

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.app.AlertDialog
import android.content.Context
import android.content.Context.CONNECTIVITY_SERVICE
import android.content.Intent
import android.content.pm.PackageManager
import android.net.ConnectivityManager
import android.net.NetworkCapabilities.NET_CAPABILITY_INTERNET
import android.net.Uri
import android.os.Build
import android.os.StrictMode
import android.os.StrictMode.ThreadPolicy
import android.provider.Settings
import android.util.Log
import androidx.annotation.NonNull
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.RequestPermissionsResultListener
import java.io.IOException
import java.net.HttpURLConnection
import java.net.URL

/** AddonsPlugin */
class AddonsPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    RequestPermissionsResultListener {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private lateinit var channel: MethodChannel
    private lateinit var activity: Activity
    private lateinit var context: Context


    @RequiresApi(Build.VERSION_CODES.M)
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "addons_plugin")
        channel.setMethodCallHandler(this)
        this.context = flutterPluginBinding.applicationContext
        this.activity.onRequestPermissionsResult(this);
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${Build.VERSION.RELEASE}")
        } else if (call.method.equals("encodeData", ignoreCase = true)) {
            if (call.hasArgument("details") && call.hasArgument("key")) {
                val details: String? = call.argument<String>("details")
                val key = call.argument<String>("key")
                try {
                    try {
                        val encStr: String
                        if (key != null) {
                            encStr =
                                details?.let { encriptionDetails().Encrypt(it, key).toString() }
                                    .toString()
                            result.success(encStr)
                        }
                    } catch (e: java.lang.Exception) {
                        e.printStackTrace()
                        result.error("UNAVAILABLE", "-" + e.message, null)
                    }
                } catch (e: java.lang.Exception) {
                    // TODO Auto-generated catch block
                    e.printStackTrace()
                    result.success("error:" + e.message)
                }
            } else {
                result.error("UNAVAILABLE", "perameters not found", null)
            }
        } else if (call.method.equals("RE_Encryption", ignoreCase = true)) {
            /*if (call.hasArgument("details") && call.hasArgument("key")) {
                val details: String? = call.argument<String>("details")
                val key: String? = call.argument<String>("key")
                try {
                    try {
                        val encStr: String
                        if (key != null) {
                            encStr = AesCipher.encrypt(details.toString(), key.toString())
                            result.success(encStr)
                        }
                    } catch (e: java.lang.Exception) {
                        e.printStackTrace()
                        result.error("UNAVAILABLE", "-" + e.message, null)
                    }
                } catch (e: java.lang.Exception) {
                    // TODO Auto-generated catch block
                    e.printStackTrace()
                    result.success("error:" + e.message)
                }
            } else {
                result.error("UNAVAILABLE", "perameters not found", null)
            }*/
            result.error("UNAVAILABLE", "perameters not found", null)
        } else if (call.method.equals("RE_Description", ignoreCase = true)) {
            /*if (call.hasArgument("details") && call.hasArgument("key")) {
                val details: String? = call.argument<String>("details")
                val key: String? = call.argument<String>("key")
                try {
                    try {
                        val encStr: String
                        if (key != null) {
                            encStr = AesCipher.decrypt(details.toString(), key.toString())
                            result.success(encStr)
                        }
                    } catch (e: java.lang.Exception) {
                        e.printStackTrace()
                        result.error("UNAVAILABLE", "-" + e.message, null)
                    }
                } catch (e: java.lang.Exception) {
                    // TODO Auto-generated catch block
                    e.printStackTrace()
                    result.success("error:" + e.message)
                }
            } else {
                result.error("UNAVAILABLE", "perameters not found", null)
            }*/
            result.error("UNAVAILABLE", "perameters not found", null)
        } else if (call.method.equals("decodeData", ignoreCase = true)) {
            if (call.hasArgument("details") && call.hasArgument("key")) {
                val details: String? = call.argument<String>("details")
                val key: String? = call.argument<String>("key")
                try {
                    assert(key != null)
                    val encStr: String? =
                        encriptionDetails().Decrypt(details.toString(), key.toString())
                    result.success(encStr)
                } catch (e: java.lang.Exception) {
                    e.printStackTrace()
                    result.error("UNAVAILABLE", "-" + e.message, null)
                }
            } else {
                result.error("UNAVAILABLE", "perameters not found", null)
            }
        } else if (call.method.equals("checkBarCode", ignoreCase = true)) {
            val code = call.argument<String>("code")!!
            val finalCode: String? = Utils_ecp().getSerialNoFromBarcode(code, true)
            result.success(finalCode)
        } else if (call.method.equals("checkBarCode16", ignoreCase = true)) {
            val code = call.argument<String>("code")!!
            val finalCode: String? = Utils_ecp().getSerialNoFromBarcode16(code, true)
            result.success(finalCode)
        } else if (call.method == "openDateSettings") {
            //result.success("Android ${android.os.Build.VERSION.RELEASE}")
            try {
                val intent = Intent(Settings.ACTION_DATE_SETTINGS)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                activity.startActivity(intent)
                result.success("done")
            } catch (e: java.lang.Exception) {
                e.printStackTrace()
                try {
                    val intent = Intent(Settings.ACTION_SETTINGS)
                    intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                    activity.startActivity(intent)
                } catch (e1: java.lang.Exception) {
                    e1.printStackTrace()
                }
                result.success("notdone")
            }
        } else if (call.method == "UniqueNumber") {
            try {
                val encStr: String? = encriptionDetails().getUniqueNumber(context)
                result.success(encStr)
            } catch (e: Exception) {
                e.printStackTrace()
                result.error("UNAVAILABLE", "-" + e.message, null)
            }
        } else if (call.method == "checkInternet") {
            //result.success("Android ${android.os.Build.VERSION.RELEASE}")
            val isNet: Boolean = isNetworkConnected(context)
            if (isNet) result.success("true") else result.success("false")
        } else if (call.method.equals("checkPermission", ignoreCase = true)) {
            Log.e("checkPermission", "checkPermission")
            if (checkAndRequestPermissions(activity)) {
                result.success("true")
            } else {
                result.success("false")
            }
        } else {
            result.notImplemented()
        }
    }

    private fun isNetworkConnected(context: Context): Boolean {
        /*val cm = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        @SuppressLint("MissingPermission") val activeNetwork = cm.activeNetworkInfo
        return activeNetwork != null &&
                activeNetwork.isConnectedOrConnecting*/
        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            isConnectedNewApi(context)
        } else {
            isConnectedOld(context)
        }
    }

    @Suppress("DEPRECATION")
    fun isConnectedOld(context: Context): Boolean {
        /*val connManager = context.getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager
        val networkInfo = connManager.activeNetworkInfo
        return networkInfo.isConnected*/
        val cm = context.getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager
        @SuppressLint("MissingPermission") val activeNetwork = cm.activeNetworkInfo
        return activeNetwork != null &&
                activeNetwork.isConnectedOrConnecting
    }


    @RequiresApi(Build.VERSION_CODES.M)
    fun isConnectedNewApi(context: Context): Boolean {
        val cm = context.getSystemService(CONNECTIVITY_SERVICE) as ConnectivityManager
        val capabilities = cm.getNetworkCapabilities(cm.activeNetwork)
        return capabilities?.hasCapability(NET_CAPABILITY_INTERNET) == true
    }

    private fun hasActiveInternetConnection(): Boolean {
        var issend: Boolean
        return if (Build.VERSION.SDK_INT <= 27) {
            val policy = ThreadPolicy.Builder().permitAll()
                .build()
            StrictMode.setThreadPolicy(policy)
            //}
            try {
                val url = URL("http://www.google.com")
                val urlc = url.openConnection() as HttpURLConnection
                urlc.setRequestProperty("User-Agent", "Test")
                urlc.setRequestProperty("Connection", "close")
                urlc.connectTimeout = 1500
                urlc.connect()
                Log.e("Utils", "Active internet connection")
                issend = urlc.responseCode == 200
            } catch (e: IOException) {
                Log.e("Utils ret false", "Error checking internet connection", e)
                issend = false
            }
            issend
        } else {
            true
        }
    }


    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        onAttachedToActivity(binding)
    }

    override fun onDetachedFromActivity() {}

    /*private boolean checkWriteExternalPermission() {
        String permission = android.Manifest.permission.WRITE_EXTERNAL_STORAGE;
        int res = registrar.context().checkCallingOrSelfPermission(permission);
        return (res == PackageManager.PERMISSION_GRANTED);
    }*/
    private fun checkAndRequestPermissions(context: Activity): Boolean {
        if (Build.VERSION.SDK_INT >= 23) {
            val locationPermission =
                ContextCompat.checkSelfPermission(context, Manifest.permission.ACCESS_FINE_LOCATION)
            val cameraPermission =
                ContextCompat.checkSelfPermission(context, Manifest.permission.CAMERA)
            //int contactsPermission = ContextCompat.checkSelfPermission(this, Manifest.permission.GET_ACCOUNTS);
//            int receiveSMSPermission = ContextCompat.checkSelfPermission(this, Manifest.permission.RECEIVE_SMS);
            //int permissionStorage = ContextCompat.checkSelfPermission(this,
            //    Manifest.permission.WRITE_EXTERNAL_STORAGE);
            val listPermissionsNeeded: MutableList<String> = ArrayList()
            if (locationPermission != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.ACCESS_FINE_LOCATION)
            }
            /**/if (cameraPermission != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.CAMERA)
            }
            /*if (contactsPermission != PackageManager.PERMISSION_GRANTED) {
                listPermissionsNeeded.add(Manifest.permission.GET_ACCOUNTS);
            }*/

            //if (receiveSMSPermission != PackageManager.PERMISSION_GRANTED) {
            //  listPermissionsNeeded.add(Manifest.permission.RECEIVE_SMS);
            //}
            if (Build.VERSION.SDK_INT <= 28) {
                val permissionStorage1 = ContextCompat.checkSelfPermission(
                    context,
                    Manifest.permission.READ_EXTERNAL_STORAGE
                )
                if (permissionStorage1 != PackageManager.PERMISSION_GRANTED) {
                    listPermissionsNeeded.add(Manifest.permission.READ_EXTERNAL_STORAGE)
                }
                val permissionStorage = ContextCompat.checkSelfPermission(
                    context,
                    Manifest.permission.WRITE_EXTERNAL_STORAGE
                )
                if (permissionStorage != PackageManager.PERMISSION_GRANTED) {
                    listPermissionsNeeded.add(Manifest.permission.WRITE_EXTERNAL_STORAGE)
                }
            }
            if (Build.VERSION.SDK_INT <= 29) {
                val callPhonePermission =
                    ContextCompat.checkSelfPermission(context, Manifest.permission.READ_PHONE_STATE)
                if (callPhonePermission != PackageManager.PERMISSION_GRANTED) {
                    listPermissionsNeeded.add(Manifest.permission.READ_PHONE_STATE)
                }
            }
            if (listPermissionsNeeded.isNotEmpty()) {
                //ActivityCompat.requestPermissions(context, listPermissionsNeeded.toArray(new String[listPermissionsNeeded.size()]), REQUEST_ID_MULTIPLE_PERMISSIONS);
                ActivityCompat.requestPermissions(
                    context,
                    listPermissionsNeeded.toTypedArray(),
                    REQUESTIDMULTIPLEPERMISSIONS
                )
                return false
            }
        }
        return true
    }

    private val REQUESTIDMULTIPLEPERMISSIONS = 1

    fun detailsPermission(
        requestCode: Int,
        permissions: Array<String>, grantResults: IntArray, context: Activity
    ): Boolean {
        //super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        val TAG = "Permission"
        Log.d(TAG, "Permission callback called-------")
        return if (requestCode == REQUESTIDMULTIPLEPERMISSIONS) {
            val perms: MutableMap<String, Int> = HashMap()
            // Initialize the map with both permissions
            perms[Manifest.permission.ACCESS_FINE_LOCATION] = PackageManager.PERMISSION_GRANTED
            perms[Manifest.permission.CAMERA] = PackageManager.PERMISSION_GRANTED


            //perms.put(Manifest.permission.GET_ACCOUNTS, PackageManager.PERMISSION_GRANTED);
            //            perms.put(Manifest.permission.RECEIVE_SMS, PackageManager.PERMISSION_GRANTED);
            if (Build.VERSION.SDK_INT <= 28) {
                perms[Manifest.permission.WRITE_EXTERNAL_STORAGE] =
                    PackageManager.PERMISSION_GRANTED
                perms[Manifest.permission.READ_EXTERNAL_STORAGE] = PackageManager.PERMISSION_GRANTED
            }
            if (Build.VERSION.SDK_INT <= 29) {
                perms[Manifest.permission.READ_PHONE_STATE] = PackageManager.PERMISSION_GRANTED
            }

            // Fill with actual results from user
            if (grantResults.isNotEmpty()) {
                for (i in permissions.indices) perms[permissions[i]] = grantResults[i]
                // Check for both permissions
                if (Build.VERSION.SDK_INT <= 28) {
                    if (perms[Manifest.permission.ACCESS_FINE_LOCATION] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.CAMERA] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.READ_EXTERNAL_STORAGE] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.WRITE_EXTERNAL_STORAGE] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.READ_PHONE_STATE] == PackageManager.PERMISSION_GRANTED) {
                        Log.d(TAG, "All permission granted")
                        true
                    } else {
                        Log.d(TAG, "Some permissions are not granted ask again ")
                        if (ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.ACCESS_FINE_LOCATION
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.CAMERA
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.READ_EXTERNAL_STORAGE
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.WRITE_EXTERNAL_STORAGE
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.READ_PHONE_STATE
                            )
                        ) {
                            false
                        } else {
                            showAlertWithOkButtonCallback(
                                2,
                                "Some Permission required for use this app.",
                                "",
                                context
                            )
                            false
                        }
                    }
                } else if (Build.VERSION.SDK_INT <= 29) {
                    if (perms[Manifest.permission.ACCESS_FINE_LOCATION] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.CAMERA] == PackageManager.PERMISSION_GRANTED && perms[Manifest.permission.READ_PHONE_STATE] == PackageManager.PERMISSION_GRANTED) {
                        Log.d(TAG, "All permission granted")
                        true
                    } else {
                        Log.d(TAG, "Some permissions are not granted ask again ")
                        if (ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.ACCESS_FINE_LOCATION
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.CAMERA
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.READ_PHONE_STATE
                            )
                        ) {
                            false
                        } else {
                            showAlertWithOkButtonCallback(
                                2,
                                "Some Permission required for use this app.",
                                "",
                                context
                            )
                            false
                        }
                    }
                } else {
                    if (perms[Manifest.permission.ACCESS_FINE_LOCATION] == PackageManager.PERMISSION_GRANTED
                        && perms[Manifest.permission.CAMERA] == PackageManager.PERMISSION_GRANTED
                    ) {
                        Log.d(TAG, "All permission granted")
                        true
                    } else {
                        Log.d(TAG, "Some permissions are not granted ask again ")
                        if (ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.ACCESS_FINE_LOCATION
                            )
                            || ActivityCompat.shouldShowRequestPermissionRationale(
                                context,
                                Manifest.permission.CAMERA
                            )
                        ) {
                            false
                        } else {
                            showAlertWithOkButtonCallback(
                                2,
                                "Some Permission required for use this app.",
                                "",
                                context
                            )
                            false
                        }
                    }
                }
            } else {
                false
            }
        } else {
            false
        }
    }

    private fun showAlertWithOkButtonCallback(
        type: Int,
        text: String,
        app_name: String,
        activity: Activity
    ) {
        val builder = AlertDialog.Builder(activity)
        builder.setTitle(app_name)
        builder.setMessage(text)
        builder.setPositiveButton("OK") { _, which ->
            if (type == 1) {
                checkAndRequestPermissions(activity)
            } else {
                val intent = Intent(
                    Settings.ACTION_APPLICATION_DETAILS_SETTINGS,
                    Uri.fromParts("package", activity.packageName, null)
                )
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                activity.startActivity(intent)
            }
        }
        builder.setCancelable(false)
        builder.show()
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>?,
        grantResults: IntArray?
    ): Boolean {


        return detailsPermission(
            requestCode, permissions as Array<String>, grantResults as IntArray,
            this.activity
        )


    }


}
